class SSVException(Exception):
    """Super class of all SSV exceptions."""


class RecordSeparatorException(SSVException):
    """Raised if the given data contains the record separator."""


class UnitSeparatorException(SSVException):
    """Raised if the given data contains the unit separator."""



from .constant import RS
from .constant import US
from .exception import RecordSeparatorException
from .exception import UnitSeparatorException


def dump(table, fp):
    fp.write(dumps(table))


def dumpf(table, path, mode='wt', encoding=None):
    with open(path, mode=mode, encoding=encoding) as f:
        dump(table, f)


def dumps(table):
    if any((RS in unit) for record in table for unit in record):
        raise RecordSeparatorException
    if any((US in unit) for record in table for unit in record):
        raise UnitSeparatorException
    return RS.join(US.join(record) for record in table)



from .constant import RS
from .constant import US


def load(fp):
    return loads(fp.read())


def loadf(path, mode='rt', encoding=None):
    with open(path, mode=mode, encoding=encoding) as f:
        return loads(f.read())


def loads(s):
    return [record.split(US) for record in s.split(RS)]



#!/usr/bin/env python

"""
test_spec
---------

Tests for the the SSV specification.
"""


import tempfile
import unittest

import ssv
from ssv.constant import RS
from ssv.constant import US


class TestSpec(unittest.TestCase):

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def check_in_out(self, table, string):
        string = string.replace("R", RS)
        string = string.replace("U", US)
        self.assertEqual(ssv.dumps(table), string)
        self.assertEqual(table, ssv.loads(string))

    def test_spec_000(self):
        self.check_in_out([[""]], "")

    def test_spec_001(self):
        self.check_in_out([[""], [""]], "R")

    def test_spec_002(self):
        self.check_in_out([["1"], [""]], "1R")

    def test_spec_003(self):
        self.check_in_out([[""], ["1"]], "R1")

    def test_spec_004(self):
        self.check_in_out([["1"], ["2"]], "1R2")

    def test_spec_005(self):
        self.check_in_out([[""], ["", ""]], "RU")

    def test_spec_006(self):
        self.check_in_out([[""], ["1", ""]], "R1U")

    def test_spec_007(self):
        self.check_in_out([[""], ["", "1"]], "RU1")

    def test_spec_008(self):
        self.check_in_out([[""], ["1", "2"]], "R1U2")

    def test_spec_009(self):
        self.check_in_out([["1"], ["2", "3"]], "1R2U3")


if __name__ == '__main__':
    import sys
    sys.exit(unittest.main())



#!/usr/bin/env python

"""
test_ssv
--------

Tests for the `ssv` module.
"""


import tempfile
import unittest

import ssv


TABLE = [['1', 'Alfa', 'V'],
         ['2', 'Bravo', 'C'],
         ['3', 'Charlie', 'C'],
         ['4', 'Delta', 'C'],
         ['5', 'Echo', 'V'],
         ['6', 'Foxtrot', 'C'],
         ['7', 'Golf', 'C'],
         ['8', 'Hotel', 'C'],
         ['9', 'India', 'V'],
         ['10', 'Juliett', 'C']]


class TestSSV(unittest.TestCase):

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_000_dump_load(self):
        with tempfile.TemporaryFile('w+t') as tmp:
            ssv.dump(TABLE, tmp)
            tmp.seek(0)
            table = ssv.load(tmp)
            self.assertEqual(table, TABLE)

    def test_001_dumpf_loadf(self):
        with tempfile.NamedTemporaryFile('w+t') as tmp:
            path = tmp.name
            ssv.dumpf(TABLE, path)
            table = ssv.loadf(path)
            self.assertEqual(table, TABLE)

    def test_002_dumps_loads(self):
        string = ssv.dumps(TABLE)
        table = ssv.loads(string)
        self.assertEqual(table, TABLE)


if __name__ == '__main__':
    import sys
    sys.exit(unittest.main())
